package gecco.client.infopanels;
import gecco.client.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import gecco.game.dke.Def; 

/**
 *  A special version of PropertyDisplay for Experiment 12.
 *  Extends ProperyDisplay only for easy implementation in PieceInfoPanel
 *
 * @author     Hans Sandstrom
 * @created    March 2, 2004
 *
 */
public class PropertyDisplayExp12 extends PropertyDisplay {

    Piece piece;
    JTextArea propertyNamesBasic;
    JTextArea propertyValuesBasic;

    JPanel namesPanel = createJPanel();
    JPanel valuesPanel = createJPanel();
    
    private boolean showPropertyValues = false;

    Game game;
    MouseAdapter mouseAdapter = new MouseAdapter(){
		public void mousePressed(MouseEvent e){
		    showPropertyValues = true;
		    updateDisplay();
		    if (game.LOG) {
			game.pieceInfoLog(piece);
		    }
		}};

    public PropertyDisplayExp12(Game game){
	this.game = game;
	propertyNamesBasic = createJTextArea();
	propertyValuesBasic = createJTextArea();

	Dimension d = namesPanel.getPreferredSize();
 	propertyNames.setMinimumSize(new Dimension(d.width, d.height*10/16));
  	propertyNames.setMaximumSize(propertyNames.getMinimumSize());
	propertyNames.setPreferredSize(propertyNames.getMinimumSize());

 	propertyValues.setMinimumSize(new Dimension(d.width, d.height*10/16));
 	propertyValues.setMaximumSize(propertyValues.getMinimumSize());
	propertyValues.setPreferredSize(propertyValues.getMinimumSize());
      	 
	// Clean up added components from PropertyDisplay()
	super.removeAll();

	// Layout for Exp12, layout is Box, from parent
 	namesPanel.add(propertyNames);
  	namesPanel.add(propertyNamesBasic);
 	valuesPanel.add(propertyValues);
  	valuesPanel.add(propertyValuesBasic);

 	add(Box.createHorizontalStrut(10));
	add(namesPanel);
	add(valuesPanel);
 	add(Box.createHorizontalStrut(10));

	addMouseListener(mouseAdapter);
	propertyNames.addMouseListener(mouseAdapter);
	propertyValues.addMouseListener(mouseAdapter);
    }

    
    private JTextArea createJTextArea(){
	JTextArea jta = new JTextArea();
        jta.setEditable(false);
	jta.setBackground(getBackground());
	jta.addMouseListener(mouseAdapter);
	return jta;
    }

    private JPanel createJPanel(){
	JPanel jp = new JPanel();
	Dimension d = getPreferredSize();
	jp.setMinimumSize(new Dimension(d.width/2-10, d.height));
	jp.setMaximumSize(jp.getMinimumSize());
	jp.setPreferredSize(jp.getMinimumSize());
	jp.addMouseListener(mouseAdapter);
	jp.setLayout(new BoxLayout(jp, BoxLayout.Y_AXIS));
	return jp;
    }


    /**
     *  Sets the piece whose information is displayed.
     *
     * @param  p  The new Piece value
     */
    public void setPiece(Piece p) {
	showPropertyValues = false;
        piece = p;
	updateDisplay();
    }

    /**
     * Used to update the information about the selected piece
     *
     */
    public void updateDisplay(){
        if (piece == null) {
            propertyNames.setText("");
            propertyValues.setText("");
	    propertyNamesBasic.setText("");
	    propertyValuesBasic.setText("");
            return;
        }
        Property pro[] = piece.getProperties();
        String names = "";
        String values = "";
        String namesBasic = "";
        String valuesBasic = "";
	for (int i = 0; i < pro.length; i++) {
	    if (Def.isPropertyBlocked(pro[i].getName())){
		if (!game.isSelectedPropertiesBlocked()){
		    names = names + pro[i].getName() + "\n";
		    if (showPropertyValues) {
			values = values + pro[i].getValue() + "\n";
		    }
		}
	    }
	    else {
		namesBasic = namesBasic + pro[i].getName() + "\n";
		valuesBasic = valuesBasic + pro[i].getValue() + "\n";
	    }
	}
	
	propertyNames.setText(names);
        propertyValues.setText(values);
        propertyNamesBasic.setText(namesBasic);
	propertyValuesBasic.setText(valuesBasic);

    }

    public Dimension getMinimumSize(){
	return this.getPreferredSize();
    }

    public Dimension getMaximumSize(){
	return this.getPreferredSize();
    }

}
