
package gecco.game.dke;

import gecco.server.automaton.*;
import gecco.server.core.UnitEvent;
import java.util.*;
import gecco.server.core.ReferenceHolder;
import java.awt.Color;
import gecco.server.replaymanager.ReplayReady;
import java.io.ObjectOutputStream;

public class AutomatonImpl extends Automaton implements ReplayReady {

    int initialState;
    boolean visited = false;
    int lastVisitor = Def.NO_FORCE;
    //      boolean blueZC = false;
    //      boolean redZC = false;
    //      Vector blueZCVector = new Vector(0);
    //      Vector redZCVector = new Vector(0);
    
    public void initialize(int initialState) {
	super.initialize(initialState);
	this.initialState = initialState;
	setColor(initialState);
    }
    
    
    protected void setColor(int terrainType) {
	switch(terrainType) {
	case Def.SEA: // Sea
	    setCurrentColor(0,0,192);
	    break;
	case Def.CLEAR: // Clear
	    setCurrentColor(255, 255, 204);
	    break;
	case Def.SWAMP: // Swamp
	    setCurrentColor(0, 128, 128); // Undefined
	    break;
	case Def.RIVER: // River
	    setCurrentColor(64, 64, 255);
	    break;
	case Def.WOODED: // Wooded
	    setCurrentColor(128, 255, 128);
	    break;
	case Def.FOREST: // Forest
	    setCurrentColor(0, 128, 0);
	    break;
	case Def.BEACH: // Beach
	    setCurrentColor(234, 230, 90);
	    break;
	case Def.RAILROAD: // Railroad
	    setCurrentColor(255, 32, 32);
	    break;
	case Def.TOWN: // Town
	    setCurrentColor(192, 192, 192);
	    break;
	case Def.VILLAGE: // Village
	    setCurrentColor(255, 128, 128);
	    break;
	case Def.ROAD_LARGE: // Road large
	    setCurrentColor(0, 0, 0);
	    break;
	case Def.ROAD_SMALL: // Road small
	    setCurrentColor(128, 128, 128);
	    break;
	case Def.ROAD_FOREST: // Road small
	    setCurrentColor(90, 80, 40);
	    break;
	case Def.BRIDGE: // Bridge
	    setCurrentColor(64, 64, 64);
	    break;
	case Def.UNDER_INDIRECT_FIRE:
	    setCurrentColor(244, 181, 30);
	    break;
	default: // Odefinierat
	    setCurrentColor(200, 0, 0);
	    break;
	}
    }
    
    
    public AutomatonReturn update(int[][] neighourStates) {
	if (getState() == Def.UNDER_INDIRECT_FIRE) {
	    setColor(initialState);
	    setState(initialState);
	}
	return new AutomatonReturnNoAction();
    }
    
    public AutomatonReturn handleEvent(AutomatonEvent event) {
	if (event.getEventName().equals(Def.getIndirectFireString())) {
	    setColor(Def.UNDER_INDIRECT_FIRE);
	    setState(Def.UNDER_INDIRECT_FIRE);
	    return new AutomatonReturnQueueSelf(Def.getTimeToShowIndirectFire(),
						new UnitEvent(Def.getIndirectFireString(), event.getFactor()));
	}
	return new AutomatonReturnNoAction();
    }
    
    public UnitEvent getUnitEventForCurrentState(String unitType) {
	return null;
    }

    public boolean isRoute(){
	if ((initialState & 64) == 64 ) {
	    return true;
	}
	else {
	    return false;   
	}
    }
    
    public boolean isVisited(){
	return visited;
    }
    
    public void setVisited(boolean b){
	visited = b;
    }

    public int getInitialState(){
	return initialState;
    }


    /**
     * Gets the value of lastVisitor
     *
     * @return the value of lastVisitor
     */
    public int getLastVisitor() {
	return this.lastVisitor;
    }

    public void setLastVisitor(int argLastVisitor, int argX, int argY){
	if (argLastVisitor != getLastVisitor()) {
	    setLastVisitor(argLastVisitor);
	    ((DKEReferee) ReferenceHolder.getReferee()).lastVisitorChanged(argLastVisitor);
	    UnitEvent ue = new UnitEvent(Def.getLastVisitorChangedString(), argLastVisitor);
	    ReferenceHolder.getActionProcessor().handleAutomatonUnitEvent(argX, argY, ue);
	    // Don't record replay data when replaying
	    if (!ReferenceHolder.inReplayMode()) {
		Object[] parameters = {new Integer(argLastVisitor), new Integer(argX), new Integer(argY)};
		saveReplayData(x, y, SET_LAST_VISITOR, parameters);
	    }
	}
    }

    /**
     * Sets the value of lastVisitor
     *
     * @param argLastVisitor Value to assign to this.lastVisitor
     */
    public void setLastVisitor(int argLastVisitor){
	this.lastVisitor = argLastVisitor;

	// Set "-markRoute" in dke arguments to visulize lastVisitor
	if (ReferenceHolder.getMarkRoute()) {
	    if (argLastVisitor == Def.BLUE_FORCE) {
		setCurrentColor(0, 0, 255);
	    }
	    else if (argLastVisitor == Def.RED_FORCE){
		setCurrentColor(255, 0, 0);
	    }
	    ReferenceHolder.getVisibilityFilter().updateAutomatonState(this.x, this.y, getCurrentColor());
	}
    }

    /**

    * Gets the value of blueZC
    *
    * @return the value of blueZC
    */
    public boolean isBlueZC() {
	return true;
	//  	return (this.blueZC && !this.redZC);
    }

    /**
     * Gets the value of redZC
     *
     * @return the value of redZC
     */
    public boolean isRedZC() {
	return true;
	//  	return (this.redZC && !this.blueZC);
    }

    public void setZC(boolean b, int force, int unitHandle){
        return;
	//  	Integer uH = new Integer(unitHandle);
	//  	if (force == Def.BLUE_FORCE) {
	//  	    if (b) {
	//  		blueZCVector.add(uH);
	//  		blueZC = true;
	//  	    }
	//  	    else {
	//  		blueZCVector.remove(uH);
	//  		if (blueZCVector.isEmpty()) {
	//  		    blueZC = false;
	//  		}
	//  	    }
	//  	}
	//  	if (force == Def.RED_FORCE) {
	//  	    if (b) {
	//  		redZCVector.add(uH);
	//  		redZC = true;
	//  	    }
	//  	    else {
	//  		redZCVector.remove(uH);
	//  		if (redZCVector.isEmpty()) {
	//  		    redZC = false;
	//  		}
	//  	    }
	//  	}

	//  	// Uncomment to visulize ZoneOfControll
	//  	if (isBlueZC()) {
	//  	    setCurrentColor(0,0,200); 
	//  	}
	//  	else if (isRedZC()) {
	//  	    setCurrentColor(200,0,0);
	//  	}
	//  	else {
	//  	    setColor(getInitialState());
	//  	}
	//  	ReferenceHolder.getVisibilityFilter().updateAutomatonState(this.x, this.y,
	//  							   getCurrentColor());
    }
	
    // implementation of gecco.server.core.ReplayReady interface

    /**
     *
     * @param param1 <description>
     * @param param2 <description>
     *
     * Format description: TimeStamp Type X Y Method Parameters
     * TimeStamp  type: double    description: game time when method was called
     * Type       type: int       description: what type of object is executing the method
     * X, Y       type: int       description: the x and y pos for the automaton
     * Method     type: int       description: which method is called, handled by the implementor
     * Parameters type: Object[]  description: parameters to the method embedded in a Object[]
     *
     *
     * OBSERVE! Since replayLog is a ObjectOutputSteam all objects that should be written must 
     * implement the java.io.Serializable interface
     *
     */
    public void saveReplayData(int function, Object[] parameters){
	saveReplayData(x, y, function, parameters);
    } 

    public void saveReplayData(int automatonPosX, int automatonPosY, int function, Object[] parameters){
	synchronized (ReferenceHolder.getReplayLog()){
	    ObjectOutputStream replayLog = ReferenceHolder.getReplayLog();
	    try {
		replayLog.writeDouble(ReferenceHolder.getClock().gameTimeNow());
		replayLog.writeInt(AUTOMATON);
		replayLog.writeInt(automatonPosX);
		replayLog.writeInt(automatonPosY);
		replayLog.writeInt(function);
		replayLog.writeObject(parameters);
		replayLog.flush();
	    } catch (Exception e) {
		e.printStackTrace();
	    }
	} 
    }
    
    private final int SET_LAST_VISITOR = 1;

    public void playReplayData(int function, Object[] parameters){
 	switch (function) {
	case SET_LAST_VISITOR :
	    int visitor = ((Integer) parameters[0]).intValue();
	    int x = ((Integer) parameters[1]).intValue();
	    int y = ((Integer) parameters[2]).intValue();
	    setLastVisitor(visitor,x,y);
	    break;
	default:
	    // Debug info
	    System.out.println("AutomatonImpl:playReplayData(int, Object[]) function " + function + " not handled!");
	    break;
	} 
    }

}


